import java.awt.*;

/**
XToolkit provides utility methods like drawing 3D boxes for buttons, displaying
messages etc.
*/
public class XToolkit
{
	//Define various color schemes for quicker color settings
	static final int COLORSCHEME_NORMAL3D   = 1;
	static final int COLORSCHEME_SHARP3D    = 2;
	static final int COLORSCHEME_SOFT3D     = 4;
	static final int COLORSCHEME_SSOFT3D    = 8;
	static final int COLORSCHEME_DIALOGEDGE = 16;

	//Types of message boxes
	static final int MESSAGE_PROGRESS = 1;
	static final int MESSAGE_STATIC   = 2;
	static final int MESSAGE_STATIC2  = 3;

	//Define edge colors to draw 3D boxes or buttons
	static final Color softEdgeColor = new Color(228, 228, 228);
	static Color brightEdgeColor = Color.white;
	static Color darkEdgeColor   = Color.gray;
	static Color shadowColor     = Color.black;

	static XMsg wMsg=null;	//reference to XMsg Object
	static Dimension wMsgD;	//Dimensions of message box
	static String wMsgText="", wMsgText2="";
	static int wMsgTextWidth=0, wMsgTextWidth2=0;

	static ProgressListener pListener=null;	//reference to progress listener which gets cancel progress msg
	static int msgType = MESSAGE_PROGRESS;

	static final Font TitleBarFont = new Font("SansSerif", Font.BOLD, 12);
	static final Font msgFont = new Font("SansSerif", Font.BOLD, 12);
	static final FontMetrics msgFM = Toolkit.getDefaultToolkit().getFontMetrics(msgFont);


	static int[] clockX1 = { 23, 20, 12, 0, -12, -20 };
	static int[] clockY1 = { 0, 12, 20, 23, 20, 12 };
	static int[] clockX2 = { 30, 26, 15, 0, -15, -26 };
	static int[] clockY2 = { 0, 15, 26, 30, 26, 15 };
	static int prevX, prevY;

	/**
	Allows the user to set bright edge color of a 3D box or button
	@param c Color of the bright edge
	*/
	public static void setBrightEdgeColor(Color c)
	{
		brightEdgeColor = c;
	}

	/**
	Allows the user to set dark edge color of a 3D box or button
	@param c Color of the dark edge
	*/
	public static void setDarkEdgeColor(Color c)
	{
		darkEdgeColor = c;
	}

	/**
	Allows the user to set shadow color of a 3D box or button
	@param c Color of the shadow
	*/
	public static void setShadowColor(Color c)
	{
		shadowColor = c;
	}

	/**
	Allows the user to set bright and dark edge colors of a 3D box or button
	according to predefined values.
	@param scheme one of the predefined color schemes Id
	*/
	public static void setColorScheme(int scheme)
	{
		switch(scheme)
		{
			case COLORSCHEME_NORMAL3D:
					brightEdgeColor = Color.white;
					darkEdgeColor   = Color.gray;
					return;

			case COLORSCHEME_SHARP3D:
					brightEdgeColor = Color.white;
					darkEdgeColor   = Color.black;
					return;

			case COLORSCHEME_SOFT3D:
					brightEdgeColor = softEdgeColor;
					darkEdgeColor   = Color.black;
					return;

			case COLORSCHEME_SSOFT3D:
					brightEdgeColor = Color.lightGray;
					darkEdgeColor   = Color.gray;
					return;

			case COLORSCHEME_DIALOGEDGE:
					brightEdgeColor = Color.lightGray;
					darkEdgeColor   = Color.black;
					return;
		}
	}

	/**
	Draws a 3D rectangle which can be either raised or engraved on the graphics
	context which is passed as a parameter.
	@param g the Graphics Context to draw upon
	@param x the left coordinate of rectangle in pixels
	@param y the top  coordinate of rectangle in pixels
	@param width the width of rectangle in pixels
	@param height the height of rectangle in pixels
	@param r if true draws a raised rectangle if false draws an engraved rectangle
	*/
	public static void draw3DBox(Graphics g, int x, int y, int width, int height, boolean r)
	{
		if(r) g.setColor(brightEdgeColor);
		else  g.setColor(darkEdgeColor);
		g.drawLine(x, y, x+width-1, y);
		g.drawLine(x, y, x, y+height-1);
		if(r) g.setColor(darkEdgeColor);
		else  g.setColor(brightEdgeColor);
		g.drawLine(x, y+height-1, x+width-1, y+height-1);
		g.drawLine(x+width-1, y, x+width-1, y+height-1);
	}

	/**
	Draws a 3D boundry around a rectangular area which can be either
	raised or engraved on the graphics context which is passed as a parameter.
	@param g the Graphics Context to draw upon
	@param x the left coordinate of rectangle in pixels
	@param y the top  coordinate of rectangle in pixels
	@param width the width of rectangle in pixels
	@param height the height of rectangle in pixels
	@param r if true draws a raised rectangle if false draws an engraved rectangle
	*/
	public static void draw3DBorder(Graphics g, int x, int y, int width, int height, int t, boolean r)
	{
		draw3DBox(g, x, y, width+2*t, height+2*t, r);
		draw3DBox(g, x+t, y+t, width, height, !r);
	}

	/**
	Draws a 3D Horizontal Seperator which can be either raised or engraved
	on the graphics context which is passed as a parameter.
	@param g the Graphics Context to draw upon
	@param x the x coordinate of starting point
	@param y the y coordinate of starting point
	@param len the length of line in pixels
	@param thickness of line in pixels
	@param r if true draws a raised line if false draws an engraved line
	*/
	public static void draw3DLineH(Graphics g, int x, int y, int len, int thickness, boolean r)
	{
		if(r) g.setColor(brightEdgeColor);
		else  g.setColor(darkEdgeColor);
		g.drawLine(x, y, x+len-1, y);
		if(r) g.setColor(darkEdgeColor);
		else  g.setColor(brightEdgeColor);
		g.drawLine(x, y+thickness, x+len-1, y+thickness);
	}

	/**
	Draws a 3D Vertical Seperator which can be either raised or engraved
	on the graphics context which is passed as a parameter.
	@param g the Graphics Context to draw upon
	@param x the x coordinate of starting point
	@param y the y coordinate of starting point
	@param len the length of line in pixels
	@param thickness of line in pixels
	@param r if true draws a raised line if false draws an engraved line
	*/
	public static void draw3DLineV(Graphics g, int x, int y, int len, int thickness, boolean r)
	{
		if(r) g.setColor(brightEdgeColor);
		else  g.setColor(darkEdgeColor);
		g.drawLine(x, y, x, y+len-1);
		if(r) g.setColor(darkEdgeColor);
		else  g.setColor(brightEdgeColor);
		g.drawLine(x+thickness, y, x+thickness, y+len-1);
	}

	/**
	This method must be called with a reference to the base container of an
	application. initMessaging adds a hidden message Panel as the first GUI
	component in the container. This makes sure that message box is the topmost
	control in the application and is always visible.
	@param c the container (usually the main Applet or Frame) to add message Panel to
	*/
	public static void initMessaging(Container c)
	{
		//create an XMsg Panel
		wMsg = new XMsg();
		wMsg.setBounds(310, 90, 340, 170);
		wMsg.setBackground(Color.lightGray);
		wMsg.setName("Billing Portal");	//to draw the title of message
		c.add(wMsg);
		//get the dimensions for later use
		wMsgD = wMsg.getSize();
	}

	/**
	Displays a message with a Progress Clock
	@param msg the message to display alongwith clock
	*/
	public static void showProgressMessage(String msg)
	{
		msgType = MESSAGE_PROGRESS;
		//get the width of the message text in pixels
		if(msg == null) wMsgTextWidth = 0;
		else { wMsgText = msg; wMsgTextWidth = msgFM.stringWidth(msg); }
		//display the message
		wMsg.setVisible(true);
		//start timer clock
		wMsg.startTimer();
	}

	/**
	This hides the progress clock message when the activity in progress is completed
	*/
	public static void hideProgressMessage()
	{
		wMsgText = "";
		wMsg.setVisible(false);
		wMsg.stopTimer();
	}

	/**
	Sets a reference to the ProgressListener object which will receive the cancel progress event
	@param pL reference to the ProgressListener
	*/
	public static void setProgressListener(ProgressListener pL) { pListener = pL; }

	/**
	Displays a single line static message
	@param msg the message String to dsiplay
	*/
	public static void showMessage(String msg)
	{
		msgType = MESSAGE_STATIC;
		if(msg == null) wMsgTextWidth = 0;
		else { wMsgText = msg; wMsgTextWidth = msgFM.stringWidth(msg); }
		wMsg.staticMessage();
		wMsg.setVisible(true);
	}

	/**
	Displays a two line static message
	@param msg1 the first message String to dsiplay
	@param msg2 the second message String to dsiplay
	*/
	public static void showMessage(String msg1, String msg2)
	{
		msgType = MESSAGE_STATIC2;
		if(msg1 == null) wMsgTextWidth = 0;
		else { wMsgText = msg1; wMsgTextWidth = msgFM.stringWidth(msg1); }
		if(msg2 == null) wMsgTextWidth2 = 0;
		else { wMsgText2 = msg2; wMsgTextWidth2 = msgFM.stringWidth(msg2); }
		wMsg.staticMessage();
		wMsg.setVisible(true);
	}

	/**
	This is called by XMsg when the user clicks the OK button for static messages
	or the Cancel button for progress messages
	*/
	public static void closeMessage()
	{
		wMsgText = "";
		wMsg.setVisible(false);
		if(msgType == MESSAGE_PROGRESS)
		{	wMsg.stopTimer();
			if(pListener != null) pListener.cancelProgress();
		}
	}

	/**
	This is called by XMsg for every paint event. The reason the paint is handled
	handled here is to let the developer modify the message drawing at one place
	and leave the XMsg class practically untouched
	@param g the graphics context of message panel
	*/
	public static void drawMsgbox(Graphics g)
	{
		setColorScheme(COLORSCHEME_DIALOGEDGE);
		draw3DBox(g, 0, 0, wMsgD.width, wMsgD.height, true);
		setColorScheme(COLORSCHEME_NORMAL3D);
		draw3DBox(g, 1, 1, wMsgD.width-2, wMsgD.height-2, true);
		g.setColor(Color.blue);
		g.fillRect(3, 3, wMsgD.width-6, 20);
		g.setColor(Color.white);
		g.setFont(TitleBarFont);
		g.drawString(wMsg.getName(), 8, 17);
		g.setColor(Color.black);
		g.setFont(msgFont);
		if(msgType == MESSAGE_PROGRESS)
			g.drawString(wMsgText, (wMsgD.width-wMsgTextWidth)/2, 120);
		else if(msgType == MESSAGE_STATIC)
		{	g.drawString(wMsgText, (wMsgD.width-wMsgTextWidth)/2, 100);
			return;
		}
		else
		{	g.drawString(wMsgText, (wMsgD.width-wMsgTextWidth)/2, 80);
			g.drawString(wMsgText2, (wMsgD.width-wMsgTextWidth2)/2, 100);
			return;
		}

		//draw the clock
		g.setColor(Color.black);
		g.drawOval(wMsgD.width/2-30, 30, 60, 60);
		int cx = wMsgD.width/2, cy=60;
		for(int i=0; i < 6; i++)
		{	g.drawLine(cx+clockX1[i], cy+clockY1[i], cx+clockX2[i], cy+clockY2[i]);
			g.drawLine(cx-clockX1[i], cy-clockY1[i], cx-clockX2[i], cy-clockY2[i]);
		}
		prevX=0;  prevY=0;
		g.setXORMode(Color.blue);
		g.setColor(Color.green);
		g.drawLine(cx, cy, cx+prevX, cy+prevY);
		g.setPaintMode();
		/*g.setColor(Color.red);
		g.drawLine(cx, cy, cx+20, cy);
		g.drawLine(cx, cy, cx+17, cy+10);
		g.drawLine(cx, cy, cx+10, cy+17);
		g.drawLine(cx, cy, cx, cy+20);*/
	}

	/**
	This updates the progress clock hand position for every tick
	@param g the graphics context of message panel
	@tick a counter for the clock ticks
	*/
	public static void updateMsgClock(Graphics g, int tick)
	{
		g.setXORMode(Color.blue);
		g.setColor(Color.green);
		int x = (int)(Math.cos(Math.PI*tick/6)*20);
		int y = (int)(Math.sin(Math.PI*tick/6)*20);
		int cx = wMsgD.width/2,  cy=60;
		g.drawLine(cx, cy, cx+prevX, cy+prevY);
		g.drawLine(cx, cy, cx+x, cy+y);
		prevX = x;  prevY = y;
		g.setPaintMode();
	}

}
