/*
 * @(#)StreamEncryptor.java	1.0
 * Copyright (c) iBasis Inc.
 *
 */
//package portal.servletbeans;

import java.io.*;
import java.net.*;

/**
This class implements a simple Stream Decryptor for reading data from an
Encrypted Stream
*/
public class EncryptedInputStream extends InputStream
{

	private static final int PACKET_LENGTH=64;
	private byte[] RandomPacket     = new byte[PACKET_LENGTH];
	private byte[] PacketScrambler  = new byte[PACKET_LENGTH];
	private int xStartPos = 23;
	private int xEndPos   = 41;

	private InputStream srcStream = null;
	private boolean isUninitialized = true;
	private int cursorPos = 0;

	/**
	The constructor creates a decryptor for an encrypted input stream.
	Initially no InputStream is associated with it and any reads on the
	stream will throw IOException. A user must call setInputStream method
	to associate an encrypted input stream to perform a read on.
	*/
	public EncryptedInputStream()
	{
	}

	/**
	This method associates an input sream (encrypted) to perform
	reads on. A user must call this method first before using the
	EncrypteInputStream objects
	@param is an InputStream which contains encrypted data to read from
	*/
	public void setInputStream(InputStream is)
	{
		srcStream = is;
		isUninitialized = true;
	}

	/**
	This method performs initializations which enable the decryption algorith
	to work. The decryption works like this: <p>The first byte contains the length
	of a randomly genertaed packed of bytes - called start scrambled bytes. After
	reading the length, the random packet is read and discarded. The next byte contains
	the length of another random packed called - end scrambled bytes. The length and
	random packed is read and discarded. Now begins the actual decryption. A packet of
	randomly generated bytes which is of length PACKET_LENGTH (usually 64) is read and
	stored. This packet was generated by EncryptedOutputStream and sent with data. After
	rading the random packet, the actual data begins. All the data bytes are encrypted by
	adding the data byte with the current random byte in the random packet. A random byte
	position counter is initialez to zero and is incremented every time a byte is encrypted.
	This repeats until all bytes are encrypted. Mathematically, the encryption is equivalent
	to auto-correlation of data with a window of PACKET_LENGTH bytes. A moderately
	sophisticated statistical analyzer can detect the encryption. To implement a more
	powerful encryptor/decryptor a complex funtion should further deteriorate the random
	packet to make statistical analysis extremely time-consuming.
	*/
	private void initStream() throws IOException
	{
		//read starting position byte
		xStartPos = srcStream.read();
		//read start scrambler bytes upto xStartPos
		srcStream.read(PacketScrambler, 0, xStartPos);
		//read ending position byte
		xEndPos = srcStream.read();
		//read end scrambler bytes upto xEndPos
		srcStream.read(PacketScrambler, 0, xEndPos);
		//read Encryption Key
		srcStream.read(RandomPacket, 0, PACKET_LENGTH);

		//set flag to indicate that stream has been initialized
		isUninitialized = false;
		cursorPos = 0;

//System.out.println("\nEncrypted Input: "+xStartPos+"  "+xEndPos);
//System.out.println("Random Sequence: ");
//for(int i=0; i < 64; i++) System.out.print((int)(RandomPacket[i])+"  ");
//System.out.println("");
	}


	/**
	Reads a decrypted byte from the input stream
	@return the decrypted byte or -1 if end of stream encountered
	*/
	public int read() throws IOException
	{
		int iValue;

		//make sure the random packet has been read
		if(isUninitialized) initStream();

		//decrypt the byte read by subtracting the current byte with the random byte
		if(cursorPos < PACKET_LENGTH)
		{
			iValue = srcStream.read() - RandomPacket[cursorPos];
			cursorPos++;
		}
		else
		{
			iValue = srcStream.read() - RandomPacket[0];
			cursorPos = 1;
		}

		return iValue;
	}

	/**
	Reads upto b.length bytes from the encrypted stream
	@param b an array of bytes to read the data into
	@return the number of bytes read or -1 if end of stream encountered
	*/
	public int read(byte b[]) throws IOException
	{
		int iValue;

		if(isUninitialized) initStream();

		iValue = srcStream.read(b);
//if(iValue > 0)
//{
//System.out.print("Encrypted Input Data: "+iValue+" :\n");
//for(int i=0; i < iValue; i++) System.out.print((int)(b[i])+"  ");
//System.out.println("");
//}
		for(int i=0; i < iValue; i++)
		{
			if(cursorPos < PACKET_LENGTH)
			{
			  b[i] -= RandomPacket[cursorPos];
			  cursorPos++;
			}
			else
			{
			  b[i] -= RandomPacket[0];
			  cursorPos = 1;
			}
		}

//if(iValue > 0)
//{
//System.out.print("Original Input Data: "+iValue+" :\n");
//for(int i=0; i < iValue; i++) System.out.print((int)(b[i])+"  ");
//System.out.println("");
//}
		return iValue;
	}

	/**
	Reads upto len bytes starting at offset off from the encrypted stream
	@param b an array of bytes to read the data into
	@param off the byte offset to start reading from
	@param len the max number of bytes to rean
	@return the number of bytes read or -1 if end of stream encountered
	*/
	public int read(byte b[], int off, int len) throws IOException
	{
		int iValue;

		if(isUninitialized) initStream();

		iValue = srcStream.read(b, off, len);

		for(int i=0; i < len; i++)
		{
			if(cursorPos < PACKET_LENGTH)
			{
			  b[off+i] -= RandomPacket[cursorPos];
			  cursorPos++;
			}
			else
			{
			  b[off+i] -= RandomPacket[0];
			  cursorPos = 1;
			}
		}

		return iValue;
	}

	/**
	Closes the underlying input stream
	*/
	public void close() throws IOException
	{
		isUninitialized = true;
		srcStream.close();
	}


	//For Testing purposes only
	/*public static void main(String[] args)
	{
		EncryptedInputStream ti = new EncryptedInputStream();
		try {
		ServerSocket ss = new ServerSocket(8081);

		byte[] data = new byte[80];

		Socket ts = ss.accept();

		long st = System.currentTimeMillis();

		ti.setInputStream(ts.getInputStream());
		int bytesRead = ti.read(data);
		System.out.println("\nData Sequence: "+bytesRead);
		for(int i=0; i < bytesRead; i++)
		{
			System.out.print((char)data[i]);
		}
		System.out.println("\nTimeTaken = "+(System.currentTimeMillis()-st));

		ts.close();
		ss.close();
		} catch (Exception e) { System.out.println("Error: "+e.toString());  }
	}
	*/
}
