/*
 * @(#)StreamEncryptor.java	1.0
 * Copyright (c) iBasis Inc.
 *
 */
//package portal.servletbeans;

import java.io.*;
import java.net.*;

/**
 * This is a simple Stream Encryptor which uses a combination of
 * random and mathematical functions to generate a sequence of
 * bytes (usually 64) which are added to the actual data to
 * achieve pseudorandom encryption
 */
public class EncryptedOutputStream extends OutputStream
{

	private java.util.Random rndGenerator = new java.util.Random();
	private static final int PACKET_LENGTH=64;
	private byte[] RandomPacket     = new byte[PACKET_LENGTH];
	private byte[] PacketScrambler  = new byte[PACKET_LENGTH];
	private byte[] PacketScrambler2 = new byte[PACKET_LENGTH];
	private int xStartPos = 23;
	private int xEndPos   = 41;

	private OutputStream destStream = null;
	private boolean isUninitialized = true;
	private int cursorPos = 0;

	/**
	The constructor creates a encryptor for an encrypted output stream.
	Initially no OutputStream is associated with it and any writes on the
	stream will throw IOException. A user must call setOutputStream method
	to associate an output stream to perform encrypted writes on.
	*/
	public EncryptedOutputStream()
	{
	}

	/**
	This method associates an output sream to perform encryption on.
	A user must call this method first before using the	EncrypteOutputStream objects
	@param os an OutputStream to perform encryption on.
	*/
	public void setOutputStream(OutputStream os)
	{
		destStream = os;
		isUninitialized = true;
	}


	/**
	This method generates a new random encryption sequence
	*/
	private void resetEncryption()
	{
		rndGenerator.setSeed(rndGenerator.nextLong());
		rndGenerator.nextBytes(RandomPacket);
		xStartPos = RandomPacket[23]/4;	//in the range of -32 to 32
		if(xStartPos < 0) xStartPos = -xStartPos;
		xEndPos = RandomPacket[PACKET_LENGTH-xStartPos-1]/4;	// range of -32 to 32
		if(xEndPos < 0) xEndPos = -xEndPos;
		rndGenerator.setSeed(rndGenerator.nextLong());
		rndGenerator.nextBytes(RandomPacket);
		rndGenerator.setSeed(rndGenerator.nextLong());
		rndGenerator.nextBytes(PacketScrambler);
		rndGenerator.nextBytes(PacketScrambler2);
	}

	/**
	This method performs initializations which enable the encryption algorith
	to work. The encryption works like this: <p>The first output byte contains the
	length of a randomly genertaed packed of bytes - called start scrambled bytes.
	After writing the length, the random packet is written. The next byte contains
	the length of another random packed called - end scrambled bytes. The length and
	random packed is written next. Now begins the actual encryption. A packet of
	randomly generated bytes which is of length PACKET_LENGTH (usually 64) is written and
	stored. After writing the random packet, the actual data is encrypted an output. All
	the data bytes are encrypted by	adding the data byte with the current random byte in the
	random packet. A random byte position counter is initialez to zero and is incremented
	every time a byte is encrypted.	This repeats until all bytes are encrypted. Mathematically,
	the encryption is equivalent to auto-correlation of data with a window of PACKET_LENGTH
	bytes. A moderately	sophisticated statistical analyzer can detect the encryption. To
	implement a more powerful encryptor/decryptor a complex funtion should further deteriorate
	the random packet to make statistical analysis extremely time-consuming.
	*/
	private void initStream() throws IOException
	{
		//generate new encryption sequence
		resetEncryption();

		//write starting position byte
		destStream.write(xStartPos);
		//write start scrambler bytes upto xStartPos
		destStream.write(PacketScrambler, 0, xStartPos);
		//write ending position byte
		destStream.write(xEndPos);
		//write end scrambler bytes upto xEndPos
		destStream.write(PacketScrambler2, 0, xEndPos);
		//write Encryption Key
		destStream.write(RandomPacket, 0, PACKET_LENGTH);

		//set flag to indicate that stream has been initialized
		isUninitialized = false;
		cursorPos = 0;

//System.out.println("\nEncrypted Output: "+xStartPos+"  "+xEndPos);
//System.out.println("Random Sequence: ");
//for(int i=0; i < 64; i++) System.out.print((int)(RandomPacket[i])+"  ");
//System.out.println("");
	}


	/**
	Writes an encrypted byte to the stream
	@param b the byte to be encrypted and written to stream.
	*/
	public void write(int b) throws IOException
	{
		if(isUninitialized) initStream();
		if(cursorPos < PACKET_LENGTH)
		{
			destStream.write((int)(b+RandomPacket[cursorPos]));
			cursorPos++;
		}
		else
		{
			destStream.write((int)(b+RandomPacket[0]));
			cursorPos = 1;
		}
	}

	/**
	Writes upto b.length bytes to the encrypted stream
	@param b an array of bytes containting the data to be written
	*/
	public void write(byte b[]) throws IOException
	{
		if(isUninitialized) initStream();
		int tLen = b.length;

//System.out.print("Original Output Data: "+tLen+" :\n");
//for(int i=0; i < tLen; i++) System.out.print((int)(b[i])+"  ");
//System.out.println("");

		for(int i=0; i < tLen; i++)
		{
			if(cursorPos < PACKET_LENGTH)
			{
			  b[i] += RandomPacket[cursorPos];
			  cursorPos++;
			}
			else
			{
			  b[i] += RandomPacket[0];
			  cursorPos = 1;
			}
		}

//System.out.print("Encrypted Output Data: "+tLen+" :\n");
//for(int i=0; i < tLen; i++) System.out.print((int)(b[i])+"  ");
//System.out.println("");

		destStream.write(b);
	}

	/**
	Writes upto len bytes starting at offset off to the encrypted stream
	@param b an array of bytes containing the data
	@param off the byte offset to start writing from
	@param len the max number of bytes to write
	*/
	public void write(byte b[], int off, int len) throws IOException
	{
		if(isUninitialized) initStream();
		for(int i=0; i < len; i++)
		{
			if(cursorPos < PACKET_LENGTH)
			{
			  b[off+i] += RandomPacket[cursorPos];
			  cursorPos++;
			}
			else
			{
			  b[off+i] += RandomPacket[0];
			  cursorPos = 1;
			}
		}
		destStream.write(b, off, len);
	}

	/**
	Closes the underlying output stream
	*/
	public void close() throws IOException
	{
		isUninitialized = true;
		destStream.close();
	}


	//For Testing purposes only
	/*public static void main(String[] args)
	{
		EncryptedOutputStream te = new EncryptedOutputStream();
		try {
		Socket ts = new Socket("localhost", 8081);

		String data = "My Name is Ash Wini...";

		long st = System.currentTimeMillis();

		te.setOutputStream(ts.getOutputStream());
		te.write(data.getBytes());
		te.close();
		ts.close();
		} catch (Exception e) { System.out.println("Error: "+e.toString());  }
	}
	*/
}
