import java.io.*;
import java.net.*;

/**
This class provides a simple HttpCommunicator which processes XML or Non-XML
requests using Java's URL class and returns the response string or documents
to the assigned receiver object.
*/
public class HttpCommunicator implements Runnable, ProgressListener
{
	private URL cURL;
	private URLConnection cURLConnection;
	private boolean doEncryption=true;
	private boolean reqCancelled=false;

	private XMLParser cParser = new XMLParser();
	private Document  inDoc, outDoc;

	private HttpListener cListener;
	private int cDataID = -1;

	private EncryptedInputStream  eIn  = new EncryptedInputStream();
	private EncryptedOutputStream eOut = new EncryptedOutputStream();


	/**
	The default constructor. A user must call setURL before using this class.
	*/
	public HttpCommunicator() { }

	/**
	Creates a HttpCommunicator which communicates with the specified URL
	@param url the URL to communicate with
	*/
	public HttpCommunicator(String url)
	{
		setURL(url);
	}

	/**
	Sets or changes the URL to communicate with
	@param url the URL to communicate with
	*/
	public void setURL(String url)
	{
		try
		{
			cURL = new URL(url);
		} catch (Exception e)
		{ System.out.println("Error: \n  HttpCommunicator error: "+e.toString()); }
	}

	/**
	sets or unsets the Communicator to use Encryption
	@param en the boolean value, which if true causes the Object to use encryption
	*/
	public void setEncryption(boolean en) { doEncryption = en; }


	/**
	Sends an Http request to the URL and returns the response string
	@param reqStr the request to be sent
	*/
	public String processRequest(String reqStr)
	{
		//if no URL associated retirn null
		if(cURL == null) return null;
		int blkSize=8192;
		byte[] xBytes = new byte[blkSize];
		StringBuffer xBuf = new StringBuffer();

		reqCancelled=false;
		try
		{	//open connection with the URL
			cURLConnection = cURL.openConnection();
			//we are using custom requests
			cURLConnection.setRequestProperty("Content-Type", "application/x-billing");
			if(reqStr != null)
			{	cURLConnection.setDoOutput(true);
				OutputStream out = cURLConnection.getOutputStream();
				//if encryption required, switch to encrypted output stream
				if(doEncryption) { eOut.setOutputStream(out); out=eOut; }
				out.write(reqStr.getBytes());
				out.close();
			}
			InputStream in = (InputStream)(cURLConnection.getContent());
			//if encryption required, switch to encrypted input stream
			if(doEncryption) { eIn.setInputStream(in); in=eIn; }
			//reads the response in blocks of size blkSize
			while(true)
			{	int nb = in.read(xBytes, 0, blkSize);
				if(nb < 0) break;
				if(nb > 0) xBuf.append(new String(xBytes, 0, nb));
			}
			in.close();
			return xBuf.toString();
		}  catch (Exception e)
		{  System.out.println("Error: \n  HttpCommunicator error: "+e.toString()); }

		return null;
	}

	/**
	Sends an XML request document to the URL and later passes on the response
	document to the HttpListener Object with dataID as Label to distinguish
	the response documents.
	@param doc the request XML document
	@param cL an Object of type HttpListener which will receive the response XML document
	@param dataID the unique identifier to be returned with document to distinguish it
	*/
	public void processXMLRequest(Document doc, HttpListener cL, int dataID)
	{
		//reset the flag which indicates cancelled request
		reqCancelled=false;
		//start a new thread which will wait for the response an we can return
		//back to the application to do some other useful things
		Thread xThread = new Thread(this);
		inDoc   = doc;
		outDoc  = null;
		cListener = cL;
		cDataID = dataID;
		//start the thread to begin communication
		xThread.start();
	}

	/**
	implements the cancelProgress method of ProgressListener interface
	*/
	public void cancelProgress() { reqCancelled=true; }

	/**
	Sends the XML output to the URL and recives and parses the response string
	as an XML document.
	*/
	public void loadXMLDocument()
	{
		//if no URL specified return null
		if(cURL == null) return;

		try
		{	cURLConnection = cURL.openConnection();
			cURLConnection.setDoOutput(true);
			cURLConnection.setRequestProperty("Content-Type", "application/x-billing");
			OutputStream out = cURLConnection.getOutputStream();
			if(doEncryption) { eOut.setOutputStream(out); out=eOut; }
			//use expressj parser to send the request document on the output stream
			cParser.outputXML(inDoc, out);
			out.close();
			InputStream in = (InputStream)(cURLConnection.getContent());
			if(doEncryption) { eIn.setInputStream(in); in=eIn; }
			//if request has not been cancelled, use expressj parser to parse
			//the response document
			if(!reqCancelled) outDoc = cParser.parseStream(in);
			in.close();
		}  catch (Exception e)
		{  System.out.println("Error: \n  HttpCommunicator error: "+e.toString()); }
	}

	/**
	This is the method which actually processes the HttpRequest
	*/
	public void run()
	{
		//send request and wait for the response
		loadXMLDocument();
		//if no cancellations send the rsponse doc to the assigned receiver
		if(!reqCancelled && cListener != null) cListener.receiveHttpData(outDoc, cDataID);
		//make sure that do documents are sent to wrong receiver if a user forgets to assign its own
		cListener = null;
	}

}
